;PROGRAM NAME: POLYGON

;PURPOSE: Draws Several Polygons not yet available in AutoCAD, R.10

; Copyright (C) 1991, CAD Consulting Services
;	Programmer: Steve McGuinness:
; Home Phone: (203) 757-7365

; Permission to use, copy, modify, and distribute this software and its
; documentation for any purpose and without fee is hereby granted.  

;HISTORY LOG:
;	8-4-90	 Steve McGuinness - Initial Coding
;	11-10-91 Steve McGuinness - Code reduction

;COMMANDS DEFINED:
;	RECT	Draws a rectangle based upon two corner points
;				and a rotation angle
;	PARA	Draws a parallelogram based upon three points
;				and a rotation angle
;	RIGHT	Draws a right triangle based upon two points,
;				an internal angle, and a rotation angle
;	ISOS	Draws an isosolees triangle based upon two points,
;				an internal angle, and a rotation angle

;MODULES DEFINED:
;	GENERR	Error Handling Routine
;	DTR	Converts Degrees to radians
;	RTD	Converts Radians to degrees

;PROGRAM CODE:

(defun GENERR (s)
	(if (/= s "Function cancelled")
		(princ (strcat "\nError: " s))
	)
	(setq tset nil)	; Free selection set
	(setq *error* olderr)
	(princ)
)

(defun DTR (A) (* PI (/ A 180.0)))
(defun RTD (R)	(* R (/ 180.0 PI)))

(defun C:RECT (/ r1pt r2pt cmde blip olderr)
	(graphscr)
	(setq
		olderr *error*
		*error* generr
		cmde (getvar "cmdecho")
		blip (getvar "blipmode")
	)
	(setvar "cmdecho" 0)
	(setvar "blipmode" 0)
	(initget 1)
	(setq r1pt (getpoint "\nPick First Corner of Rectangle... "))
	(initget (+ 1 32))
	(setq r2pt (getcorner r1pt "\nPick Opposite Corner of Rectangle... "))
	(cond
		((equal r1pt r2pt) (prompt "\nPoints Cannot be Identical."))
		(t (command "pline"
				r1pt (list (car r1pt)(cadr r2pt))
				r2pt (list (car r2pt)(cadr r1pt)) "C"
			)
			(prompt "\nPick or Enter angle of rotation: ")
			(command "rotate" "Last" "" r1pt pause)
		)
	)
	(setvar "cmdecho" cmde)
	(setvar "blipmode" blip)
	(setq *error* olderr)
	(princ)
)

(defun C:PARA (/ p1pt p2pt p3pt cmde blip olderr)
	(graphscr)
	(setq
		olderr *error*
		*error* generr
		cmde (getvar "cmdecho")
		blip (getvar "blipmode")
	)
	(setvar "cmdecho" 0)
	(setvar "blipmode" 0)
	(initget 1)
	(setq p1pt (getpoint "\nPick Corner of Paralellogram... "))
	(initget 1)
	(setq p2pt (getpoint p1pt "\nPick First Adjacent Corner..."))
	(if (equal p1pt p2pt)
		(prompt "\nPoints Cannot be Identical.")
		(progn
			(grdraw p1pt p2pt -1)
			(initget 1)
			(setq p3pt (getpoint p1pt "\nPick Second Adjacent Corner..."))
			(cond
				((equal p1pt p3pt) (prompt "\nPoints Cannot be Identical."))
				((equal p2pt p3pt) (prompt "\nPoints Cannot be Identical."))
				(t
					(command "pline" p1pt p2pt
						(polar p2pt (angle p1pt p3pt) (distance p1pt p3pt))
						p3pt "c"
					)
					(prompt "\nPick or Enter angle of rotation: ")
					(command "rotate" "Last" "" p1pt "Reference"
						p1pt p2pt pause
					)
				)
			)
		)
	)
	(setvar "cmdecho" cmde)
	(setvar "blipmode" blip)
	(setq *error* olderr)
	(princ)
)

(defun C:RIGHT (/ a b c cmde blip orth anga angd aa ab da db olderr)
	(graphscr)
	(setq
		olderr *error*
		*error* generr
		cmde (getvar "cmdecho")
		blip (getvar "blipmode")
		orth (getvar "orthomode")
		angb (getvar "angbase")
		angd (getvar "angdir")
	)
	(setvar "cmdecho" 0)
	(setvar "blipmode" 0)
	(initget 1)
	(setq a (getpoint "\nPick Corner of Triangle... "))
	(setvar "orthomode" 1)
	(initget (+ 1 32))
	(setq b (getpoint a "\nPick Tip of Triangle... "))
	(cond
		((equal a b) (prompt "\nPoints Cannot be Identical."))
		(t
			(grdraw a b -1)
			(setvar "angbase" (angle b a))	; radians
			(setvar "angdir" 1)		; clockwise
			(setvar "orthomode" 0)
			(initget (+ 1 2 3))
			(setq aa 0)
			(while (or (<= aa 0) (>= aa (dtr 90)))
				(prompt "\nAngle must be > 0 degrees and < 90 degrees.")
				(setq aa (getangle b "\nPick or Enter Interior Angle: "))
				(setq aa (- (* 2.0 PI) aa))
			)
			(setq
				ab (- (/ PI 2) aa)
				da (abs (distance a b))
				db (* (/ da (sin ab)) (sin aa))
				ac (+ (angle a b) (/ PI 2))
				c (polar a ac db)
			)
			(setvar "angbase" angb)
			(setvar "angdir" angd)
			(command "pline" a b c "c")
			(prompt "\nPick or Enter angle of rotation: ")
			(command "rotate" "Last" "" a "Reference" a b pause)
		)
	)
	(setvar "cmdecho" cmde)
	(setvar "blipmode" blip)
	(setvar "orthomode" orth)
	(setq *error* olderr)
	(princ)
)

(defun C:ISOS (/ a b c cmde blip orth aa ab da olderr)
	(graphscr)
	(setq
		olderr *error*
		*error* generr
		cmde (getvar "cmdecho")
		blip (getvar "blipmode")
		orth (getvar "orthomode")
	)
	(setvar "cmdecho" 0)
	(setvar "blipmode" 0)
	(initget 1)
	(setq a (getpoint "\nPick Corner of Triangle... "))
	(setvar "orthomode" 1)
	(initget (+ 1 32))
	(setq b (getpoint a "\nPick Tip of Triangle... "))
	(cond
		((equal a b) (prompt "\nPoints Cannot be Identical."))
		(t
			(grdraw a b -1)
			(setq angb (getvar "angbase"))	; radians
			(setq angd (getvar "angdir"))		; 0 or 1
			(setvar "angbase" (angle b a))	; radians
			(setvar "angdir" 1)		; clockwise
			(setvar "orthomode" 0)
			(initget (+ 1 2 3))
			(setq aa 0)
			(while (or (<= aa 0) (>= aa (dtr 90)))
				(prompt "\nAngle must be > 0 degrees and < 90 degrees.")
				(setq aa (getangle b "\nPick or Enter Interior Angle: "))
				(setq aa (- (* 2.0 PI) aa))
			)
			(setq ab (- PI (* 2 aa)))
			(setq da (abs (distance a b)))
			(setvar "angbase" angb)
			(setvar "angdir" angd)
			(setq ac (+ (angle a b) ab))
			(setq c (polar a ac da))
			(command "pline" a b c "c")
			(prompt "\nPick or Enter angle of rotation: ")
			(command "rotate" "Last" "" a "Reference" a b pause)
		)
	)
	(setvar "cmdecho" cmde)
	(setvar "blipmode" blip)
	(setvar "orthomode" orth)
	(setq *error* olderr)
	(princ)
)
